<template>
    <div class="p-4">
      <h1 class="text-2xl font-bold">Build Inhabitants Run Code-CMD <<< Chat </h1>
    </div>
    <div class="p-4">
      <h1 class="text-2xl font-bold"> [ IFRAME LIVE EXEC WIDGET HERE ] </h1>
    </div>

    <div class="p-4">
      <h1 class="text-2xl font-bold">Test Inhabitants Track & new RFC >>> Chat </h1>
    </div>
    <div>
      <ChatbotToggle />
      <Chatbot />
    </div>
  </template>
  
  <script setup>
  import ChatbotToggle from '~/components/ChatbotToggle.vue'
  import Chatbot from '~/components/ChatBot.vue'
  </script>
  